/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"
#include "dlfcn.h"

#ifndef RTLD_GLOBAL /* freebsd doesn't support this flag, blegh */
#define RTLD_GLOBAL 0
#endif

void dso_init()
{
    xmlnode next;

    _etherx_dso(manager_send, dso_transport); /* tell libetherx stuff */

    next = xmlnode_get_firstchild(config);
    while(next != NULL)
    {
        if(j_strcmp(xmlnode_get_name(next),"dso") == 0 && xmlnode_get_attrib(next,"init") != NULL)
            dso_load(next);
        next = xmlnode_get_nextsibling(next);
    }
}

/* code for dynamically loading transports */

void dso_load(xmlnode x)
{
    void (*init)(void);
    void (*hack)(void *send, void *reg);
    void *tp_so;
    char *defso;

    defso = xmlnode_get_data(xmlnode_get_firstchild(x));

    log_debug(ZONE,"dso_load: %s",defso);

    tp_so = dso_open(defso);
    if(tp_so == NULL)
    {
        log_error("etherxd","DSO load of %s failed",defso);
        return;
    }

    hack = dso_sym(tp_so,"_etherx_dso");
    if(hack == NULL)
    {
        log_error("etherxd","DSO init of %s failed",defso);
        return;
    }
    (hack)(manager_send, dso_transport); /* XXX hack to work around dlopen issues, GRRR */

    init = dso_sym(tp_so,xmlnode_get_attrib(x,"init"));
    if(init == NULL)
    {
        log_error("etherxd","DSO init of %s failed",defso);
        return;
    }

    (init)(); /* tell transport to start up */
}

void *dso_open(char *so)
{
    char *dlerr;
    void *so_h;

    if(so == NULL)
        return NULL;

    so_h = dlopen(so,RTLD_LAZY|RTLD_GLOBAL);
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        log_debug(ZONE,"dso_open(%s) failed: %s",so, dlerr);
        return NULL;
    }

    return so_h;
}

void *dso_sym(void *so, char *func)
{
    char *dlerr;
    void *func_h;

    if(so == NULL || func == NULL)
        return NULL;

    func_h = dlsym(so,func);
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        log_debug(ZONE,"dso_sym(%s) failed: %s",func, dlerr);
        return NULL;
    }

    return func_h;
}

void dso_transport(ehandler eh)
{
    transports = ehandler_list_add(transports,eh);
}
