/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"
#include <getopt.h>


/* code for an etherx daemon
    - loads config file
    - error and connection log file
    - DSO loads transports
    - listens on sockets and manages connections
    - routes epackets between connections and transports
*/

int main(int argc, char *argv[])
{
    pth_event_t sigevt;
    sigset_t set;
    int sig, help;
    char *log, *cfgfile = NULL;
    struct option options[] =
    {
        {"config",1,0,'c'},
        {0,0,0,0}
    };

    help = opterr = 0;
    while((sig = getopt_long(argc,argv,"c:D",options,NULL)) > 0)
    {
        switch(sig)
        {
            case 'c':
                cfgfile = strdup(optarg);
                break;
            case 'D':
                etherx_debug();
                break;
            default:
                help = 1;
        }
    }

    if(help)
    {
        printf("Usage:\netherxd [-c etherx.xml] [-D]\n");
        exit(0);
    }

    if(cfgfile == NULL)
        cfgfile = strdup(SYSCONFDIR "/etherx.xml");

    config_load(cfgfile);

    log = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(xmlnode_get_tag(config,"etherx"),"error")));
    if(log == NULL)
        log_init(LOGSTDERR, NULL);
    else if(j_strcmp(log,"syslog") == 0)
        log_init(LOGSYSLOG, "etherxd");
    else
        log_init(LOGFILE, log);

    logger_init();
    if(manager_init() <= 0)
    {
        printf("Unable to bind to port 5269.  Either check your <listen></listen> directive(s) for validity or verify that an existing etherxd isn't running (use 'netstat -ln' and look for ':5269')\n");
        exit(0);
    }
    dso_init();

    sigemptyset(&set);
    sigaddset(&set, SIGHUP);
    sigaddset(&set, SIGINT);
    sigaddset(&set, SIGTERM);
    pth_sigmask(SIG_UNBLOCK, &set, NULL);
    sigevt = pth_event(PTH_EVENT_SIGS, &set, &sig);

    /* main thread, wait on kill signals and react appropriately */
    log_warn("etherxd","server started");
    pth_wait(sigevt);
    log_warn("etherxd","received kill signal, shutting down");
    /* XXX need to shut down gracefully, empty buffers, etc */
    pth_yield(NULL); /* one last chance to make sure everyone is done */
    pth_kill();
    return 0;
}

