/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherx.h"

void (*_etherx_send_internal)(epacket e) = NULL;
void (*_etherx_reg_internal)(ehandler e) = NULL;
char *_etherx_host = NULL;
char *_etherx_secret = NULL;

void _etherx_dso(void *send, void *reg)
{
    log_debug(ZONE,"_etherx_dso");
    _etherx_send_internal = send;
    _etherx_reg_internal = reg;
    log_debug(ZONE,"_etherx_reg_internal: %d",_etherx_reg_internal);
}

void etherx_init(char *secret, char *host)
{
    _etherx_host = strdup(host);
    _etherx_secret = strdup(secret);
    log_debug(ZONE,"_etherx_reg_internal: %d",_etherx_reg_internal);
}

ehandler _ehandler_new(char *host, char *namespace)
{
    pool p;
    ehandler e;

    log_debug(ZONE,"_ehandler_new(%s,%s)",host,namespace);

    p = pool_new();
    e = pmalloc(p, sizeof(_ehandler));
    e->host = pstrdup(p, host);
    e->namespace = pstrdup(p, namespace);
    e->p = p;
    e->h = NULL;
    e->t = NULL;
    e->type = EHANDLER_ERROR;

    return e;
}

ehandler ehandler_new(etherx_handler h, char *host, char *namespace)
{
    ehandler e;

    e = _ehandler_new(host, namespace);
    e->h = h;

    log_debug(ZONE,"_etherx_reg_internal: %d",_etherx_reg_internal);
    if(_etherx_reg_internal != NULL)
    {
        e->type = EHANDLER_DSO;
        (_etherx_reg_internal)(e);
    }else{
        e->type = EHANDLER_SOCKET;
        _etherx_connect(e, host, namespace);
    }
    return e;
}

void ehandler_send(ehandler eh, xmlnode x, char *to)
{
    epacket ep;

    if(eh == NULL || x == NULL || to == NULL)
    {
        log_warn("libetherx","received illegal outgoing data");
        return;
    }

    ep = epacket_new(x, to, eh->host, eh->namespace);

    if(eh->type == EHANDLER_SOCKET)
    {
        log_debug(ZONE,"ehandler_send: socket %s",xmlnode2str(x));
        xmlstream_write(eh->t, epacket_serialize(ep));
        return;
    }
    if(eh->type == EHANDLER_DSO)
    {
        log_debug(ZONE,"ehandler_send: dso %s",xmlnode2str(x));
        (_etherx_send_internal)(ep);
        return;
    }

    /* if we get here it's because the type is EHANDLER_ERROR from the socket disconnecting, bounce back to the handler */
    epacket_error(ep,"delivery failure");
    (*eh->h)(ep);
}

