/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherx.h"

extern char *_etherx_host;
extern char *_etherx_secret;

void _etherx_connect(ehandler eh, char *host, char *namespace)
{
    tstream ts;
    xmlnode x;

    eh->t = xmlstream_connect(5269, _etherx_host, (xmlstream_onNode)_etherx_receive);
    ts = (tstream)(eh->t->data);
    ts->data_aux1 = (void *)eh; /* so that _etherx_receive can access the handler info */
    x = xmlstream_header(namespace, _etherx_host, host);
    xmlnode_put_attrib(x, "xmlns:etherx", ETHERX_NAMESPACE);
    xmlnode_put_attrib(x, "etherx:secret", _etherx_secret);
    xmlstream_write_header(eh->t, x);
}

void _etherx_receive(thread t, xmlnode x, int flag)
{
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);
    epacket ep;

    switch(flag)
    {
        case XMLSTREAM_ROOT:
            log_debug(ZONE,"got root tag from etherx!");
            ts->flag_aux1 = 1; /* flag that we've connected successfully at least once */
            eh->type = EHANDLER_SOCKET; /* make sure we re-set our type if it was ERROR */
            /* ignore the root tag content, transports don't use it */
            break;
        case XMLSTREAM_BOUNCE:
            ep = epacket_new(xmlnode_dup(x), NULL, NULL, eh->namespace);
            log_error("libetherx","bouncing data intended for etherx");
            epacket_error(ep,"delivery failure");
            (*eh->h)(ep);
            break;
        case XMLSTREAM_NODE:
            log_debug(ZONE,"got node from etherx!");
            if(j_strcmp(xmlnode_get_name(x),"stream:error") == 0)
            {
                log_error("libetherx","connection to etherxd failed: %s",xmlnode_get_data(xmlnode_get_firstchild(x)));
                raise(SIGTERM);
                break;
            }
            ep = epacket_new(xmlnode_dup(x), eh->host, NULL, eh->namespace);
            (*eh->h)(ep);
            break;
        case XMLSTREAM_INIT:
            pth_cleanup_push(_etherx_cleanup, (void *)t);
            break;
    }
}

void _etherx_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);

    if(ts->flag_aux1 && eh->type != EHANDLER_ERROR)
    {
        eh->type = EHANDLER_ERROR;
        log_warn("libetherx","Connection to etherxd failed, attempting to reconnect...");
        pth_sleep(2); /* give etherxd a chance to restart if it was killed */
        _etherx_connect(eh, eh->host, eh->namespace);
        return;
    }

    eh->type = EHANDLER_ERROR;
    log_error("libetherx","etherxd connection failed, forcing exit");
    raise(SIGTERM);
}
